package at.ac.tuwien.sepm.assignment.individual.unit.persistence;

import static org.junit.jupiter.api.Assertions.*;

import at.ac.tuwien.sepm.assignment.individual.persistence.impl.HorseFileDao;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.context.ActiveProfiles;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

@SpringBootTest
@ActiveProfiles("test")
public class HorseFileDaoTest {
    @Value("${spring.upload.path}")
    private String FILE_BASE_PATH;
    @Autowired
    HorseFileDao horseFileDao;

    @Test
    @DisplayName("Creating a file with the correct type should result in a saved file")
    public void savingFile_correctType_shouldWriteFile() throws FileNotFoundException, IOException {
        File image = new File("src/test/resources/at/ac/tuwien/sepm/assignment/individual/integration/horse.jpg");
        horseFileDao.save(new MockMultipartFile("file", image.getName(), MediaType.IMAGE_JPEG_VALUE,  new FileInputStream(image)));
        File savedImage = new File(FILE_BASE_PATH + "horse.jpg");
        assertTrue(savedImage.exists());
        assertTrue(savedImage.isFile());
    }

    @Test
    @DisplayName("Creating a file with the correct type should result in a saved file")
    public void savingFile_incorrectType_shouldThrowIOException() throws FileNotFoundException, IOException {
        File image = new File("src/test/resources/at/ac/tuwien/sepm/assignment/individual/integration/horse.jpg");
        assertThrows(IOException.class, () -> horseFileDao.save(new MockMultipartFile("file", image.getName(), MediaType.TEXT_HTML_VALUE,  new FileInputStream(image))));
    }

    @AfterEach
    public void cleanup() {
        new File(FILE_BASE_PATH + "horse.jpg").delete();
    }
}
